import { Injector } from '@angular/core';
import { Location} from '@angular/common';
import { Router,UrlSerializer } from '@angular/router';
import { AuthConfig, OAuthService, OAuthErrorEvent, OAuthStorage  } from 'angular-oauth2-oidc';
import { AppConfig } from "./app.config";
import {ItemTypeService} from '../services/itemtype.service';
import { IAuthconfigFactory } from './authconfigFactory';



export function appConfigFactory(injector:Injector, appConfig: AppConfig, oauthService: OAuthService, authconfigFactory:IAuthconfigFactory,authStorage:OAuthStorage,itemtypeService:ItemTypeService): () => Promise<any> {
  return (): Promise<any> => {
    return new Promise((resolve,reject) => {
      appConfig.load().then(() => {     
        oauthService.events.subscribe((event) => {
          console.debug(event.type);
          if (event.type == 'token_error' || event.type == 'silent_refresh_timeout') {
            let e = event as OAuthErrorEvent;
            let p = e.params as any;
            if (event.type == 'silent_refresh_timeout' || (p.error && p.error == 'login_required')) {
              let router = injector.get(Router);
              console.debug("Session expired");
              router.navigate(['loggedout'], { queryParams: { redirectTo: router.url } });
            }          
          }
        });
        oauthService.configure(authconfigFactory.getAuthConfig(appConfig));
        oauthService.setStorage(authStorage);
        oauthService.setupAutomaticSilentRefresh();
        let location = injector.get(Location);
        let router = injector.get(Router);
        oauthService.loadDiscoveryDocument().then(() => {
          oauthService.tryLoginCodeFlow().then(() => {
            router.navigateByUrl(decodeURIComponent(oauthService.state));
          });    
        })
      }).then(() => {
        itemtypeService.load(appConfig).then(() => resolve()).catch(() => reject());
      });
    });
  }
}

